import logging
import requests

from kidibox.exceptions import ApiError


logger = logging.getLogger(__name__)


class Session(requests.Session):
    def __init__(self, verify=None, token=None):
        super(Session, self).__init__()
        if verify is not None:
            self.verify = verify
        self.token = token

    def is_authenticated(self):
        return (self.token is not None)

    def raise_for_status(self, response):
        if 400 <= response.status_code < 500:
            raise ApiError(
                "Error {0}".format(response.status_code), response=response)
        else:
            response.raise_for_status()

    def get(self, url, **kwargs):
        logger.debug(
            "Making GET request for %s (verify=%s)", url, self.verify)
        response = super(Session, self).get(url, **kwargs)
        self.raise_for_status(response)
        return response

    def post(self, url, **kwargs):
        logger.debug(
            "Making POST request for %s (verify=%s)", url, self.verify)
        response = super(Session, self).post(url, **kwargs)
        self.raise_for_status(response)
        return response



import logging

from kidibox.session import Session
from kidibox import api


logger = logging.getLogger(__name__)


class Client(api.AuthApiMixin, api.TorrentsApiMixin, api.DownloadApiMixin):
    session_class = Session

    def __init__(self, url, username, password, verify=None):
        self.session = self.session_class(verify=verify)
        self.url = url
        self.username = username
        self.password = password

    def __repr__(self):
        return 'Client(url={url!r}, username={username!r})'\
            .format(url=self.url, username=self.username)

    def _make_request(self, method, path, **kwargs):
        if 'headers' not in kwargs:
            kwargs['headers'] = {}
        if self.is_authenticated():
            kwargs['headers']['Authorization'] = "Bearer " + self.session.token
        url = "{base_url}{path}".format(base_url=self.url, path=path)
        caller = getattr(self.session, method)
        return caller(url, **kwargs)

    def _get(self, path, **kwargs):
        return self._make_request('get', path, **kwargs)

    def _post(self, path, **kwargs):
        return self._make_request('post', path, **kwargs)

    def is_authenticated(self):
        return self.session.is_authenticated()



import requests.exceptions


class ApiError(requests.exceptions.HTTPError):
    def __init__(self, *args, **kwargs):
        super(ApiError, self).__init__(*args, **kwargs)
        self.returned_data = self.response.json()

    def __str__(self):
        if 'message' in self.returned_data:
            return '{0} {1}: {2} ({3})'.format(
                self.returned_data['statusCode'], self.returned_data['error'],
                self.returned_data['message'], self.response.url)
        else:
            return '{0} {1} ({2})'.format(
                self.returned_data['statusCode'], self.returned_data['error'],
                self.response.url)



import logging

from kidibox.client import Client


__author__ = 'Cecile Tonglet'
__version__ = '0.1.1'


def set_stream_logger(
        name='kidibox', level=logging.DEBUG, format_string=None):
    """
    Add a stream handler for the given name and level to the logging module.
    By default, this logs all kidibox messages to ``stdout``.
        >>> import kidibox
        >>> kidibox.set_stream_logger('kidibox.client', logging.INFO)
    :type name: string
    :param name: Log name
    :type level: int
    :param level: Logging level, e.g. ``logging.INFO``
    :type format_string: str
    :param format_string: Log message format
    """
    if format_string is None:
        format_string = "%(asctime)s %(name)s [%(levelname)s] %(message)s"

    logger = logging.getLogger(name)
    logger.setLevel(level)
    handler = logging.StreamHandler()
    handler.setLevel(level)
    formatter = logging.Formatter(format_string)
    handler.setFormatter(formatter)
    logger.addHandler(handler)

def connect(*args, **kwargs):
    return Client(*args, **kwargs)


logging.getLogger('kidibox').addHandler(logging.NullHandler())



import logging

from .decorators import requires_authentication


logger = logging.getLogger(__name__)


class TorrentsApiMixin(object):
    @requires_authentication
    def list(self):
        return self._get('/torrents').json()

    @requires_authentication
    def create(self, link):
        return self._post('/torrents/link', json={
            'link': link,
        }).json()

    @requires_authentication
    def get(self, id):
        return self._get('/torrents/{0}'.format(id)).json()

    @requires_authentication
    def get_token(self, id, file_index):
        return self._get(
            '/torrents/{0}/files/{1}/token'.format(id, file_index)).json()



import logging


logger = logging.getLogger(__name__)


class AuthApiMixin(object):
    def register(self):
        return self._post('/register', json={
            'username': self.username,
            'password': self.password,
        }).json()

    def authenticate(self):
        response = self._post('/authenticate', json={
            'username': self.username,
            'password': self.password,
        }).json()
        self.session.token = response['token']
        return response



import logging
import re


logger = logging.getLogger(__name__)
re_filename = re.compile(r"filename=(.+)")


class DownloadApiMixin(object):
    def download(self, token, **kwargs):
        response = self._get('/download/{0}'.format(token), stream=True)
        return (
            re_filename.search(
                response.headers['content-disposition']).group(1),
            response.iter_content(**kwargs))



#!/usr/bin/env python

"""
distutils/setuptools install script.
"""
import os
import re
import sys

from setuptools import setup, find_packages


ROOT = os.path.dirname(__file__)
VERSION_RE = re.compile(r'''__version__ = ['"]([0-9.]+)['"]''')


requires = [
    'requests >= 2.6',
]


def get_version():
    init = open(os.path.join(ROOT, 'kidibox', '__init__.py')).read()
    return VERSION_RE.search(init).group(1)


setup(
    name='kidibox-py',
    version=get_version(),
    description='A Python library to interact with kidibox-api',
    long_description=open('README.md').read(),
    author='Cecile Tonglet',
    url='https://github.com/kidibox/kidibox-py',
    scripts=[],
    packages=find_packages(exclude=['tests*']),
    include_package_data=True,
    install_requires=requires,
    setup_requires=['pytest-runner'],
    tests_require=['pytest'],
    license="MIT",
    classifiers=[
        'Development Status :: 1 - Planning',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
)



from functools import wraps


def requires_authentication(func):
    @wraps(func)
    def wrapper(client, *args, **kwargs):
        if not client.is_authenticated():
            client.authenticate()
        return func(client, *args, **kwargs)
    return wrapper
